﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity.ModelConfiguration.Conventions.Configuration;
using System.Data.Entity.ModelConfiguration.Configuration.Types;

namespace CrmDemoCodeFirst {

  #region DefaultSchemaConvention

  public class DefaultSchemaConvention : IConfigurationConvention<Type, EntityTypeConfiguration> {

    string defaultSchema;

    public DefaultSchemaConvention(string defaultSchema) {

      if (String.IsNullOrWhiteSpace(defaultSchema))
        throw new ArgumentException("defaultSchema");

      this.defaultSchema = defaultSchema;
    }

    #region IConfigurationConvention<PropertyInfo,EntityTypeConfiguration> Members

    void IConfigurationConvention<Type, EntityTypeConfiguration>.Apply(Type memberInfo, Func<EntityTypeConfiguration> configuration) {

      EntityTypeConfiguration cfg = configuration();
      string tableName = cfg.EntitySetName;
      if(String.IsNullOrEmpty(tableName))
        tableName = memberInfo.Name;
      if(cfg.IsTableNameConfigured) {
        var attribute = memberInfo.GetCustomAttributes(false).OfType<System.ComponentModel.DataAnnotations.TableAttribute>().FirstOrDefault();
        if(attribute != null && !string.IsNullOrEmpty(attribute.TableName)) {
          if(!string.IsNullOrEmpty(attribute.SchemaName))
            return;
          tableName = attribute.TableName;
        }
      }
      cfg.ToTable(tableName, this.defaultSchema);
    }

    #endregion
  }

  #endregion

}
