﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Entity;
using System.Data.Entity.ModelConfiguration;
using System.ComponentModel.DataAnnotations;

namespace CrmDemoCodeFirst {

  #region MyDbContext

  /// <summary>
  /// 
  /// </summary>
  public class MyDbContext : DbContext {

    protected override void OnModelCreating(ModelBuilder modelBuilder) {

      // -------------------------------------------------------------
      // ColumnTypeCasingConvention should be removed for dotConnect for Oracle.  
      // This option is obligatory only for SqlClient.  
      // Turning off ColumnTypeCasingConvention isn't necessary for  dotConnect for MySQL, PostgreSQL, and SQLite.  
      //modelBuilder.Conventions.Remove<System.Data.Entity.ModelConfiguration.Conventions.Edm.Db.ColumnTypeCasingConvention>();
      // -------------------------------------------------------------

      // -------------------------------------------------------------
      // If you don't want to create and use EdmMetadata table for monitoring 
      // the correspondence between the current model and table structure  
      // created in a database, turn off IncludeMetadataConvention:
      //modelBuilder.Conventions.Remove<System.Data.Entity.Database.IncludeMetadataConvention>();
      // -------------------------------------------------------------

      // -------------------------------------------------------------
      // We have defined autoincrement columns in the primary key and non-nullable columns 
      // using DataAnnotation attributes. Similarly, the same can be done with Fluent mapping:

      //modelBuilder.ComplexType<AddressType>();

      //modelBuilder.Entity<Product>().HasKey(p => p.ProductID);
      //modelBuilder.Entity<Product>().Property(p => p.ProductID)
      //    .HasDatabaseGenerationOption(DatabaseGenerationOption.Identity);
      //modelBuilder.Entity<Product>().Property(p => p.ProductName)
      //    .IsRequired()
      //    .HasMaxLength(50);


      //modelBuilder.Entity<ProductCategory>().HasKey(p => p.CategoryID);
      //modelBuilder.Entity<ProductCategory>().Property(p => p.CategoryID)
      //    .HasDatabaseGenerationOption(DatabaseGenerationOption.Identity);
      //modelBuilder.Entity<ProductCategory>().Property(p => p.CategoryName)
      //    .IsRequired()
      //    .HasMaxLength(20);

      //modelBuilder.Entity<OrderDetail>().HasKey(p => new { p.OrderID, p.ProductID });

      //modelBuilder.Entity<Order>().HasKey(o => o.OrderID);
      //modelBuilder.Entity<Order>().Property(o => o.OrderID)
      //    .HasDatabaseGenerationOption(DatabaseGenerationOption.Identity);
      //modelBuilder.Entity<Order>().Property(o => o.OrderDate)
      //    .IsRequired();

      //modelBuilder.Entity<Company>().HasKey(c => c.CompanyID);
      //modelBuilder.Entity<Company>().Property(c => c.CompanyID)
      //    .HasDatabaseGenerationOption(DatabaseGenerationOption.Identity);
      //modelBuilder.Entity<Company>().Property(c => c.CompanyName)
      //    .IsRequired()
      //    .HasMaxLength(40);
      //modelBuilder.Entity<Company>().Property(c => c.Web)
      //    .HasMaxLength(100);
      //modelBuilder.Entity<Company>().Property(c => c.Email)
      //    .HasMaxLength(50);

      //modelBuilder.Entity<PersonContact>().HasKey(pc => pc.ContactID);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.ContactID)
      //    .HasDatabaseGenerationOption(DatabaseGenerationOption.Identity);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.Title)
      //    .HasMaxLength(8);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.FirstName)
      //    .HasMaxLength(50);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.MiddleName)
      //    .HasMaxLength(50);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.LastName)
      //    .HasMaxLength(50);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.HomePhone)
      //    .HasMaxLength(25);
      //modelBuilder.Entity<PersonContact>().Property(pc => pc.MobilePhone)
      //    .HasMaxLength(25);

      //  As a solution for correct EdmMetadata mapping to the needed database schema.  
      // We have created and we are using our own convention - DefaultSchemaConvention.  
      // It allows you to avoid a necessity to define mapping  for all entities 
      // using Table attributes or using ToTable() fluent mapping.  
      // It uses schema name for Oracle:
      modelBuilder.Conventions.Add(new DefaultSchemaConvention("TEST"));
      // -------------------------------------------------------------

      // -------------------------------------------------------------
      // By default when a class wasn't explicitly mapped using the Table attribute  
      // or fluent mapping then Model-First uses default mapping, suitable for SQL Server,  
      // but ineligible for other databases, table will have a "dbo" prefix,  
      // so table names will be the following - "dbo.TableName".
      // An alternative solution is fluent mapping:
      //modelBuilder.Entity<Product>().ToTable("Product", "TEST");
      //modelBuilder.Entity<ProductCategory>().ToTable("ProductCategory", "TEST");
      //modelBuilder.Entity<OrderDetail>().ToTable("Order Details", "TEST");
      //------------------------------------------------------------------
      // If IncludeMetadataConvention was turned off then it will be enough to define 
      // Table attributes for all entities or use .ToTable() in fluent mapping.  
      // If IncludeMetadataConvention was turned on, then "dbo"."EdmMetadata" table 
      // will be used and it is impossible to change the schema name in easy way.  
      // As a solution for correct EdmMetadata mapping  to the needed database schema
      // the DefaultSchemaConvention class is used.

      // It is necessary to use a database name as a parameter in MySQL:
      //modelBuilder.Conventions.Add(new DefaultSchemaConvention("test"));
      
      // Schema name is used as a parameter in PostgreSQL (usually it is "public"):
      //modelBuilder.Conventions.Add(new DefaultSchemaConvention("public"));

      // It is necessary to specify MAIN in SQLite:
      //modelBuilder.Conventions.Add(new DefaultSchemaConvention("MAIN"));
      // -------------------------------------------------------------
    }

    public DbSet<Product> Products { get; set; }
    public DbSet<ProductCategory> ProductCategories { get; set; }
    public DbSet<OrderDetail> OrderDetails { get; set; }
    public DbSet<Order> Orders { get; set; }
    public DbSet<Company> Companies { get; set; }
    public DbSet<PersonContact> PersonContacts { get; set; }
  }

  #endregion
}
